from pydantic import BaseModel

class ChatMessage(BaseModel):
    role: str = 'user'
    content: str

class ChatRequest(BaseModel):
    model: str = 'gpt-3.5-turbo'
    messages: list[ChatMessage]

class Choice(BaseModel):
    index: int
    message: ChatMessage
    finish_reason: str

class Usage(BaseModel):
    prompt_tokens: int
    completion_tokens: int
    total_tokens: int

class ChatResponse(BaseModel):
    id: str
    object: str
    created: int
    choices: list[Choice]
    usage: Usage

class ImageRequest(BaseModel):
    prompt: str
    n: int = 1
    size: str = '1024x1024'
    response_format: str = 'url'

class Url(BaseModel):
    url: str

class ImageResponse(BaseModel):
    created: int
    data: list[Url]

class Error(BaseModel):
    code: str | None
    message: str
    param: str | None
    type: str

class ErrorResponse(BaseModel):
    error: Error



from simple_openai import SimpleOpenai

# Get the API key from the file
try:
    with open("open_ai_key.txt", "r") as f:
        api_key = f.read().strip()
except FileNotFoundError:
    print("No API key found. Please create a file called 'api_key.txt' and paste your API key in it.")
    exit()

def main():
    # Create the client
    client = SimpleOpenai(api_key)

    # Create a list of results for the chat responses and the image response
    results = [
        client.get_image_url("A cat"),
        client.get_chat_response("Where is Alaska?"),
        client.get_chat_response("Where is Moscow?"),
        client.get_chat_response("Where is London?"),
        client.get_chat_response("Where is Georgia?"),
        client.get_chat_response("Where is Beijing?"),
        client.get_chat_response("Where is Sydney?"),
        client.get_chat_response("Where is Melbourne?"),
        client.get_chat_response("Where is New York?"),
        client.get_chat_response("Where is Wellington?"),
        client.get_chat_response("Where is Birmingham?"),
        client.get_chat_response("Where is Manchester?"),
    ]

    # Get the results
    for result in results:
        # Print the result
        if result.success:
            # Print the message
            print(f'Success: {result.message}')
        else:
            # Print the error
            print(f'Error: {result.message}')

if __name__ == "__main__":
    # Run the main function
    main()



"""Simple OpenAI API wrapper

The is the synchronous version of the Simple OpenAI API wrapper which uses the [`requests`](https://requests.readthedocs.io/en/latest/) library.

If you wish to use the async version, you should use the [AsyncSimple OpenAI API wrapper](/simple_openai/async_simple_openai/) instead.
"""

import requests

from . import constants
from .models import open_ai_models

from .responses import SimpleOpenaiResponse

class SimpleOpenai:
    """Simple OpenAI API wrapper

    This class implements the Simple OpenAI API wrapper.

    To use this class, you need to have an OpenAI API key. You can get one from [Openai](https://platform.openai.com).

    Args:
        api_key (str): Your OpenAI API key

    !!!Example
        ```python
        from simple_openai import SimpleOpenai

        def main():
            # Create the client
            client = SimpleOpenai(api_key)

            # Create tasks for the chat response and the image response
            result = client.get_chat_response("Hello, how are you?")

            # Print the result
            if result.success:
                # Print the message
                print(f'Success: {result.message}')
            else:
                # Print the error
                print(f'Error: {result.message}')

            result = client.get_image_url("A cat")

            # Print the result
            if result.success:
                # Print the message
                print(f'Success: {result.message}')
            else:
                # Print the error
                print(f'Error: {result.message}')

        if __name__ == "__main__":
            # Run the main function
            main()
        ```
    """
    def __init__(self, api_key: str) -> None:
        self._headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {api_key}'
        }

    def get_chat_response(self, prompt: str) -> SimpleOpenaiResponse:
        """Get a chat response from OpenAI

        Args:
            prompt (str): The prompt to use for the chat response

        Returns:
            SimpleOpenaiResponse: The chat response, the value of `success` should be checked before using the value of `message`

        !!!Example
            ```python
            from simple_openai import SimpleOpenai

            def main():
            
                # Create the client
                client = SimpleOpenai(api_key)

                # Get the chat response
                response = client.get_chat_response("Hello, how are you?")

                # Check if the request was successful
                if response.success:
                    # Print the chat response
                    print(f'Chat response: {response.message}')
                else:
                    # Print the error message
                    print(f'Error: {response.message}')

            if __name__ == "__main__":
                main()
            ```

        """

        # Create the request body
        chat_message = open_ai_models.ChatMessage(content=prompt)
        request_body = open_ai_models.ChatRequest(messages=[chat_message])

        # Send the request
        response = requests.post(constants.FULL_CHAT_URL, json=request_body.dict(), headers=self._headers)

        # Check the status code
        if response.status_code == requests.codes.OK:
            # Parse the response body
            response_body = open_ai_models.ChatResponse.parse_raw(response.text)

            # Create the response
            response = SimpleOpenaiResponse(True, response_body.choices[0].message.content)
        else:
            # Parse the error response body
            response_body = open_ai_models.ErrorResponse.parse_raw(response.text)

            # Create the response
            response = SimpleOpenaiResponse(False, response_body.error.message)

        # Return the response
        return response

    def get_image_url(self, prompt: str) -> SimpleOpenaiResponse:
        """Get an image response from OpenAI

        Args:
            prompt (str): The prompt to use

        Returns:
            SimpleOpenaiResponse: The image response, the value of `success` should be checked before using the value of `message`

        !!!Example
            ```python
            from simple_openai import SimpleOpenai

            def main():
            
                # Create the client
                client = AsyncSimpleOpenai(api_key)

                # Get the image response
                response = client.get_image_url("A cat")

                # Check if the request was successful
                if response.success:
                    # Print the image URL
                    print(f'Image Generated Successfully, it can be found at {response.message}')
                else:
                    # Print the error message
                    print(f'Image Generation Failed, Error: {response.message}')

            if __name__ == "__main__":
                main()
            ```
        """
            
        # Create the request body
        request_body = open_ai_models.ImageRequest(prompt=prompt)

        # Send the request
        response = requests.post(constants.FULL_IMAGE_URL, json=request_body.dict(), headers=self._headers)

        # Check the status code
        if response.status_code == requests.codes.OK:
            # Parse the response body
            response_body = open_ai_models.ImageResponse.parse_raw(response.text)

            # Create the response
            response = SimpleOpenaiResponse(True, response_body.data[0].url)
        else:
            # Parse the error response body
            response_body = open_ai_models.ErrorResponse.parse_raw(response.text)

            # Create the response
            response = SimpleOpenaiResponse(False, response_body.error.message)

        # Return the response
        return response



from .simple_openai import SimpleOpenai
from .async_simple_openai import AsyncSimpleOpenai
from .responses import SimpleOpenaiResponse

__all__ = [
    'SimpleOpenai',
    'AsyncSimpleOpenai',
    'SimpleOpenaiResponse',
]



"""Async Simple OpenAI API wrapper

The is the async version of the Simple OpenAI API wrapper which uses the [`aiohttp`](https://docs.aiohttp.org/en/stable/index.html) library.

It is intended for use with asyncio applications.  If you are not using asyncio, you should use the [Simple OpenAI API wrapper](/simple_openai/simple_openai/) instead.
"""

import aiohttp

from . import constants
from .models import open_ai_models

from .responses import SimpleOpenaiResponse

class AsyncSimpleOpenai:
    """Async Simple OpenAI API wrapper

    This class implements the Async Simple OpenAI API wrapper.

    To use this class, you need to have an OpenAI API key. You can get one from [Openai](https://platform.openai.com).

    Args:
        api_key (str): Your OpenAI API key

    !!!Example
        ```python
        from simple_openai import AsyncSimpleOpenai
        import asyncio

        async def main():
            # Create the client
            client = AsyncSimpleOpenai(api_key)

            # Create tasks for the chat response and the image response
            tasks = [
                client.get_chat_response("Hello, how are you?"),
                client.get_image_url("A cat"),
            ]

            # Wait for the tasks to complete
            for task in asyncio.as_completed(tasks):
                # Get the result
                result = await task

                # Print the result
                if result.success:
                    # Print the message
                    print(f'Success: {result.message}')
                else:
                    # Print the error
                    print(f'Error: {result.message}')

        if __name__ == "__main__":
            # Run the main function
            asyncio.run(main())
        ```
    """
    def __init__(self, api_key: str) -> None:
        self._headers = {
            'Content-Type': 'application/json',
            'Authorization': f'Bearer {api_key}'
        }

    async def get_chat_response(self, prompt: str) -> SimpleOpenaiResponse:
        """Get a chat response from OpenAI

        Args:
            prompt (str): The prompt to use for the chat response

        Returns:
            SimpleOpenaiResponse: The chat response, the value of `success` should be checked before using the value of `message`

        !!!Example
            ```python
            from simple_openai import AsyncSimpleOpenai
            import asyncio

            async def main():
            
                # Create the client
                client = AsyncSimpleOpenai(api_key)

                # Get the chat response
                response = await client.get_chat_response("Hello, how are you?")

                # Check if the request was successful
                if response.success:
                    # Print the chat response
                    print(f'Chat response: {response.message}')
                else:
                    # Print the error message
                    print(f'Error: {response.message}')

            if __name__ == "__main__":
                asyncio.run(main())
            ```

        """

        # Create the request body
        chat_message = open_ai_models.ChatMessage(content=prompt)
        request_body = open_ai_models.ChatRequest(messages=[chat_message])

        # Open a session
        async with aiohttp.ClientSession(headers=self._headers, base_url=constants.BASE_URL) as session:
            # Send the request
            async with session.post(constants.CHAT_URL, json=request_body.dict()) as response:
                # Check the status code
                if response.status == 200:
                    # Parse the response body
                    response_body = open_ai_models.ChatResponse.parse_raw(await response.text())

                    # Create the response
                    response = SimpleOpenaiResponse(True, response_body.choices[0].message.content)
                else:
                    # Parse the error response body
                    response_body = open_ai_models.ErrorResponse.parse_raw(await response.text())

                    # Create the response
                    response = SimpleOpenaiResponse(False, response_body.error.message)

                # Return the response
                return response

    async def get_image_url(self, prompt: str) -> SimpleOpenaiResponse:
        """Get an image response from OpenAI

        Args:
            prompt (str): The prompt to use

        Returns:
            SimpleOpenaiResponse: The image response, the value of `success` should be checked before using the value of `message`

        !!!Example
            ```python
            from simple_openai import AsyncSimpleOpenai
            import asyncio

            async def main():
            
                # Create the client
                client = AsyncSimpleOpenai(api_key)

                # Get the image response
                response = await client.get_image_url("A cat")

                # Check if the request was successful
                if response.success:
                    # Print the image URL
                    print(f'Image Generated Successfully, it can be found at {response.message}')
                else:
                    # Print the error message
                    print(f'Image Generation Failed, Error: {response.message}')

            if __name__ == "__main__":
                asyncio.run(main())
            ```
        """
            
        # Create the request body
        request_body = open_ai_models.ImageRequest(prompt=prompt)

        # Open a session
        async with aiohttp.ClientSession(headers=self._headers, base_url=constants.BASE_URL) as session:
            # Send the request
            async with session.post(constants.IMAGE_URL, json=request_body.dict()) as response:
                # Check the status code
                if response.status == 200:
                    # Parse the response body
                    response_body = open_ai_models.ImageResponse.parse_raw(await response.text())

                    # Create the response
                    response = SimpleOpenaiResponse(True, response_body.data[0].url)
                else:
                    # Parse the error response body
                    response_body = open_ai_models.ErrorResponse.parse_raw(await response.text())

                    # Create the response
                    response = SimpleOpenaiResponse(False, response_body.error.message)

                # Return the response
                return response
